SystemOrganization addCategory: #'Mondrian-ComplexShape'!

MONodeShape subclass: #MOFormsShape
	instanceVariableNames: 'builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-ComplexShape'!

!MOFormsShape commentStamp: 'Alexandre.Bergel 2/4/2009 10:54' prior: 0!
A MOFormsShape is the shape created by a FormsBuilder.

Instance Variables
	builder:		<Object>
	debug:		<Object>
	isChildrenShape:		<Object>

builder
	- xxxxx

debug
	- xxxxx

isChildrenShape
	- xxxxx
!

!MOFormsShape classSide methodsFor: 'instance creation' stamp: ' 23/10/08 11:55'!
on: aBuilder
	
	^(self new)
		builder: aBuilder;
		yourself! !

!MOFormsShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:15'!
acceptVisitor: aVisitor
^aVisitor visitFormsShape: self.! !

!MOFormsShape methodsFor: 'accessing' stamp: 'Alexandre Bergel 8/11/2010 08:56'!
builder
	
	^builder ifNil: [ builder := MOFormsBuilder new ]! !

!MOFormsShape methodsFor: 'initialize-release' stamp: ' 23/10/08 11:55'!
builder: aBuilder
	
	builder := aBuilder! !

!MOFormsShape methodsFor: 'private' stamp: ' 23/10/08 11:55'!
builderAt: aX and: aY
	"Utility method that makes other methods easier to read."
	
	^self builder at: aX and: aY! !

!MOFormsShape methodsFor: 'testing' stamp: 'Alexandre Bergel 8/9/2010 17:52'!
canHandleChildren
	
	^ self collectAllShapes anySatisfy: [:s | s isChildShape ]! !

!MOFormsShape methodsFor: 'drawing' stamp: 'AlexandreBergel 4/9/2011 14:58'!
canvasTranslationOf: shape For: anElement
	"Return translation step of the canvas before drawing anElement using shape"
	"This step corresponds to the offset of a shape within a formshape. 
	 Note that shape may not necessary be a child shape, and shape is not always self"
	"However, the element shape is always self"
	"This suggests that this method may be located on MONode directly"

	self assert: [ anElement shape == self ].
"	self assert: [ self builder shapes includes: shape ]."

	^ (anElement shapeBoundsAt: shape) origin + anElement bounds origin negated! !

!MOFormsShape methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 4/28/2011 23:37'!
collectAllShapes
	"Return the list of shapes I may have under myself. Overriden for form shapes"

	| allShapes |
	allShapes := OrderedCollection new.
	allShapes add: self.
	self builder shapes
		do: [:shape | allShapes addAll: shape collectAllShapes].	"We also need the decorations"
	^allShapes! !

!MOFormsShape methodsFor: 'public' stamp: 'AlexandreBergel 4/9/2011 08:54'!
computeBoundsFor: graphElement
	
	| myWidth myHeight myWidths myHeights v positionsX positionsY |
	"Calculate the grid"
	myWidths := self builder widthsFor: graphElement.
	myHeights := self builder heightsFor: graphElement.	"Position and resize the components"

	myWidth := myWidths
		inject: 0
		into: [:sum :nextElement | sum + nextElement].
	myHeight := myHeights
		inject: 0
		into: [:sum :nextElement | sum + nextElement].
		
	v := 0.
	positionsX := myWidths copy.
	1 to: positionsX size do: [:i | positionsX at: i put: v. v := v + (myWidths at: i) ].

	v := 0.
	positionsY := myHeights copy.
	1 to: positionsY size do: [:i | positionsY at: i put: v. v := v + (myHeights at: i) ].

	self builder shapes do: [:shape | 
		shape parent: self.

		graphElement shapeBoundsAt: shape shape put:
			(self builder boundsOf: shape shape for: graphElement )].
	
	"I am not sure about this value (0@0), before it was 'aFigure origin'"
	^Rectangle
			origin: (0@0)
			extent: (myWidth@myHeight) asIntegerPoint 
! !

!MOFormsShape methodsFor: 'public' stamp: 'Alexandre.Bergel 8/30/2009 13:18'!
computeExtentFor: anElement 
	^ self extentFor: anElement! !

!MOFormsShape methodsFor: 'initialize-release' stamp: 'Alexandre.Bergel 2/21/2009 14:01'!
defaultBounds

	^ Rectangle left: 0 top: 0 width: 10 height: 10! !

!MOFormsShape methodsFor: 'private' stamp: 'Alexandre Bergel 6/17/2010 11:46'!
extentFor: anElement
	^ (self boundsFor: anElement) extent! !

!MOFormsShape methodsFor: 'public' stamp: 'Alexandre Bergel 2/18/2010 01:31'!
heightFor: anElement
	^ (self boundsFor: anElement) height
! !

!MOFormsShape methodsFor: 'testing' stamp: 'Alexandre Bergel 6/9/2010 19:31'!
isCached 
	^ false! !

!MOFormsShape methodsFor: 'testing' stamp: 'Alexandre Bergel 8/10/2010 10:05'!
isFormsShape
	^ true! !

!MOFormsShape methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 8/2/2011 17:32'!
positionOfChildrenForOwner: aGraphElement
	"This method is complex. We should not delegate to the owner"
	
	"We check if this form shape is included within another "
	self assert: [ aGraphElement owner notNil ] description: 'an owner has to be specified'.
	^ aGraphElement owner shape isFormsShape 
		ifTrue: [ 
			| outterChildrenShape |
			outterChildrenShape := (aGraphElement owner shape builder shapes collect: #component) 	
										detect: #isChildShape ifNone: [ nil ].
			self assert:  [ outterChildrenShape notNil ] description: 'A children shape is missing'.
			outterChildrenShape absoluteOriginForChildrenElement: aGraphElement owner ]
		ifFalse: [ aGraphElement owner absoluteBounds origin  ]
	! !

!MOFormsShape methodsFor: 'public' stamp: 'AlexandreBergel 9/5/2010 11:33'!
translateBoundsForFigure: anElement by: aPoint context: aDictionary
	
	aDictionary
		at: self
		put: ((aDictionary at: self) translatedBy: aPoint).
	self builder shapes
		do:
			[:adaptor | 
			adaptor component translateBoundsForFigure: anElement by: aPoint context:
					aDictionary ].	"Move the nested figures"

! !

!MOFormsShape methodsFor: 'public' stamp: 'Alexandre Bergel 2/18/2010 01:31'!
widthFor: anElement
	^ (self boundsFor: anElement) width
! !

MOLayout subclass: #MOFormsLayout
	instanceVariableNames: 'builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-ComplexShape'!

!MOFormsLayout classSide methodsFor: 'instance creation' stamp: ' 24/4/08 15:59'!
on: aBuilder
	
	^(self new)
		builder: aBuilder;
		yourself! !

!MOFormsLayout methodsFor: 'accessing' stamp: ' 24/4/08 15:59'!
builder
	
	^builder! !

!MOFormsLayout methodsFor: 'initialize-release' stamp: ' 24/4/08 15:59'!
builder: aBuilder
	
	builder := aBuilder! !

!MOFormsLayout methodsFor: 'private - utils' stamp: ' 24/4/08 15:59'!
builderAt: aX and: aY
	"Utility method that makes other methods easier to read."
	
	^self builder at: aX and: aY! !

!MOFormsLayout methodsFor: 'hook' stamp: 'AlexandreBergel 4/7/2011 12:47'!
doExecute: aGraphElement
	
	| myWidths myHeights |
	"Calculate the grid"
	myWidths := self builder widthsFor: aGraphElement.
	myHeights := self builder heightsFor: aGraphElement.	"Position and resize the components"
	self builder runWithWidths: myWidths heights: myHeights.	"Move the figures to the calculated positions"
	self builder graphElements do: [:figure | figure install]! !

!MOGraphElement methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 4/26/2011 17:49'!
computeChildrenOrigin
	"Return the origin of the graph element according to the position of the children shape."
	| builder childrenShape ownerPosition elementOrigin deltaX deltaY childrenPosition |

	self 
		assert: [ self owner notNil ] 
		description: 'Not an inner node'.	
	self 
		assert: [ self owner shape isFormsShape ] 
		description: 'The shape of the owner is not a form shape'.

	builder := self getAssociatedBuilder.
	childrenShape := self computeParentShape.

	ownerPosition := builder boundsOf: childrenShape for: self owner.
	^ ownerPosition origin.
	! !

!MOGraphElement methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 4/26/2011 17:49'!
computeParentShape
	| builder childrenShape |

	self 
		assert: [ self owner notNil ] 
		description: 'Not an inner node'.	
	self 
		assert: [ self owner shape isFormsShape ] 
		description: 'The shape of the owner is not a form shape'.

	builder := self owner shape builder.
	childrenShape := builder shapes detect: [ :s | s shape isChildShape ].
	
	^ childrenShape shape! !

!MOGraphElement methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 4/26/2011 17:13'!
getAssociatedBuilder

	self assert: [ self hasParentShape ].
	^  self owner shape builder! !

!MOGraphElement methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 4/26/2011 17:12'!
getAssociatedBuilderFor: aGraphElement

	self assert: [ self hasParentShape ].
	^  self owner shape builder! !

!MOGraphElement methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 5/4/2011 14:27'!
hasBuilderAChildShape
	"True if my builder has a child shape"

	| builder |
	builder := self owner shape builder.
	^ builder shapes anySatisfy: [ :s | s shape isChildShape ]! !

!MOGraphElement methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 5/4/2011 14:27'!
hasParentShape
	"True if I have a parent shape (i.e., if I am an inner node, my owner is a form shape and my builder has a child shape)"

	"Must be inner node"
	self owner notNil
		ifFalse: [ ^ false ].	
	
	"shape of the owner is not a form shape"
	self owner shape isFormsShape
		ifFalse: [ ^ false ].	
		
	"the builder must contain at least one child shape"
	^ self hasBuilderAChildShape! !

!MOShape methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 4/28/2011 23:37'!
collectAllShapes
	"Return the list of shapes I may have under myself. Overriden for form shapes"

	 ^OrderedCollection with: self! !

!MOShape methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 4/28/2011 18:52'!
isShape
	"I am  a Mondrian shape. This method is intended to be overriden"

	^true! !

!Rectangle class methodsFor: '*mondrian-complexshape' stamp: 'Alexandre.Bergel 2/5/2009 18:45'!
left: x top: y width: w height: h
	^ self origin: (x@y) corner: (x + w) @ (y + h)! !

!Rectangle class methodsFor: '*mondrian-complexshape' stamp: 'lr 9/17/2006 23:07'!
origin: aPoint width: aWidth height: aHeight
	^ self origin: aPoint corner: (aPoint x + aWidth) @ (aPoint y + aHeight)! !

!MOShapeSelector methodsFor: '*Mondrian-ComplexShape-composite' stamp: 'AlexandreBergel 4/24/2011 12:32'!
a: shape withNameOnLeft: aBlock
	| builder |
	builder := MOFormsBuilder new.
	builder column; pref; center; column; fill.
	builder row; fill.
	builder x: 1 y: 1 add: ((MORectangleShape new withoutBorder; withText) text: aBlock).
	builder x: 2 y: 1 add: shape.
	^ view shape: builder shape! !

!MOShapeSelector methodsFor: '*Mondrian-ComplexShape-composite' stamp: 'AlexandreBergel 4/24/2011 12:33'!
a: shape withNameOnTop: aBlock
	| builder |
	builder := MOFormsBuilder new.
	builder column; fill; center.
	builder row; fill; row; fill.
	builder x: 1 y: 1 add: ((MORectangleShape new withoutBorder; withText) text: aBlock).
	builder x: 1 y: 2 add: shape.
	^ view shape: builder shape! !

!MOShapeSelector methodsFor: '*Mondrian-ComplexShape-composite' stamp: 'TudorGirba 4/18/2011 21:31'!
detailsWithTitle: aBlock
	| builder |
	builder := MOFormsBuilder new.
	builder column; pref; center;
			row; fill; 
			row; fill.
	builder x:1 y:1 add: (MORectangleShape new text: aBlock; withoutBorder).
	builder x:1 y:2 add: MOChildrenShape new.
	^ view shape: builder asShape! !

!MOShapeSelector methodsFor: '*Mondrian-ComplexShape' stamp: 'Alexandre Bergel 2/28/2010 23:30'!
form: aOneArgBlock

	| builder |
	builder := MOFormsBuilder new.
	aOneArgBlock value: builder.
	^ view shape: builder asShape! !

!MOShapeSelector methodsFor: '*Mondrian-ComplexShape-composite' stamp: 'TudorGirba 1/19/2011 15:59'!
umlClass
	^ self umlClassWithTitle: #printString! !

!MOShapeSelector methodsFor: '*Mondrian-ComplexShape-composite' stamp: 'TudorGirba 1/19/2011 15:58'!
umlClassWithTitle: aBlock
	| builder |
	builder := MOFormsBuilder new.
	builder column; center; fill; pref; grow.
	builder 
		row; center; pref; fill; grow; 
		row; center; fill; pref; grow.
	builder x: 1 y: 1 add: (MORectangleShape new text: aBlock).
	builder x: 1 y: 2 add: MORectangleShape new .
	builder x: 1 y: 2 add: MOChildrenShape new.
	^ view shape: builder asShape! !

Object subclass: #MOFormsAdaptor
	instanceVariableNames: 'gridX component bounds columnSpan gridY rowSpan'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-ComplexShape'!

MOFormsAdaptor subclass: #MOFigureAdaptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-ComplexShape'!

!MOFigureAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:07'!
height: aNumber
	
	bounds height: aNumber! !

!MOFigureAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 4/9/2011 09:06'!
heightFor: agraphElem
	^ self height! !

!MOFigureAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 10/6/2010 11:15'!
initializeBounds

	bounds := component bounds! !

!MOFigureAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 10/6/2010 11:14'!
install
	
	component translateTo: bounds origin! !

!MOFigureAdaptor methodsFor: 'accessing' stamp: 'Alexandre.Bergel 3/29/2009 19:37'!
translateBy: aPoint
	
	bounds := (bounds origin + aPoint) extent: (bounds extent)! !

!MOFigureAdaptor methodsFor: 'accessing' stamp: 'Alexandre.Bergel 3/29/2009 19:35'!
translateTo: aPoint
	
	self translateBy: aPoint - bounds origin
"	component container notNil
		ifTrue:
			[self
				translateBy:
					aPoint - bounds origin + component container childArea origin]
		ifFalse: [self translateBy: aPoint - bounds origin]
	"! !

!MOFigureAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:07'!
width: aNumber
	
	bounds width: aNumber! !

!MOFormsAdaptor classSide methodsFor: 'instance creation' stamp: ' 23/10/08 11:11'!
on: aComponent
	
	^self new component: aComponent! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
bounds
	
	^bounds! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
columnSpan
	
	^columnSpan! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
columnSpan: aW
	
	columnSpan := aW! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
component
	
	^component! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
component: aComponent
	
	component := aComponent! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
gridX
	
	^gridX! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
gridX: aNumber
	
	gridX := aNumber! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
gridY
	
	^gridY! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
gridY: aNumber
	
	gridY := aNumber! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
height
	
	^bounds height! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
height: aNumber
	
	self subclassResponsibility! !

!MOFormsAdaptor methodsFor: 'delegation' stamp: 'Alexandre.Bergel 8/30/2009 13:27'!
heightFor: anElement
	^ component heightFor: anElement! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
install
	
	self subclassResponsibility! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
rowSpan
	
	^rowSpan! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
rowSpan: aH
	
	rowSpan := aH! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: 'Alexandre.Bergel 2/21/2009 14:42'!
shape
	
	^component! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
translateBy: aPoint
	
	self subclassResponsibility! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
translateTo: aPoint
	
	self subclassResponsibility! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
width
	
	^bounds width! !

!MOFormsAdaptor methodsFor: 'accessing' stamp: ' 23/10/08 11:11'!
width: aNumber
	
	self subclassResponsibility! !

!MOFormsAdaptor methodsFor: 'delegation' stamp: 'AlexandreBergel 4/9/2011 09:05'!
widthFor: anElement
	"strange"
	^ self width! !

MOFormsAdaptor subclass: #MOShapeAdaptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-ComplexShape'!

!MOShapeAdaptor methodsFor: 'testing' stamp: 'Alexandre Bergel 8/9/2010 17:43'!
canHandleChildren
	"read only"
	^component canHandleChildren! !

!MOShapeAdaptor methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 4/28/2011 23:37'!
collectAllShapes
	"Return the list of shapes I may have under myself"

	^component collectAllShapes! !

!MOShapeAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2010 09:36'!
height: aNumber! !

!MOShapeAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 4/9/2011 08:52'!
heightFor: graphElement

	^ (component class == MOChildrenShape)
		ifTrue: [ (component extentFor: graphElement) y ]
		ifFalse: [ component computeHeightHavingNoChildrenFor: graphElement ]
! !

!MOShapeAdaptor methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 5/18/2011 15:57'!
install
	
	self error: 'not appropriate'! !

!MOShapeAdaptor methodsFor: 'testing' stamp: 'AlexandreBergel 4/28/2011 18:52'!
isShape
	"I am a Mondrian shape. This method is intended to be overriden"

	^true! !

!MOShapeAdaptor methodsFor: 'accessing' stamp: 'jlr 11/11/2008 15:37'!
parent: aParentShape 
	component parent: aParentShape! !

!MOShapeAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 5/18/2011 15:57'!
translateBy: aPoint
	self error: 'not appropriate'! !

!MOShapeAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 5/18/2011 15:56'!
translateTo: aPoint
	self error: 'not appropriate'! !

!MOShapeAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2010 09:37'!
width: aNumber! !

!MOShapeAdaptor methodsFor: 'accessing' stamp: 'AlexandreBergel 4/9/2011 08:51'!
widthFor: graphElement
"	| anElement |
	anElement := graphElement attributeAt: #shapeAdaptor.
	self assert: [ anElement notNil ] description: 'The layout of the view must first be performed'.
"
	^ (component class == MOChildrenShape)
		ifTrue: [ (component extentFor: graphElement) x ]
		ifFalse: [ component computeWidthHavingNoChildrenFor: graphElement ]
! !

Object subclass: #MOFormsBuilder
	instanceVariableNames: 'columnAlignment columnResizeBehaviour rowAlignment rowResizeBehaviour components columnDefaultSize rowDefaultSize alignment resizeBehaviour defaultSize shapes graphElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-ComplexShape'!

!MOFormsBuilder commentStamp: 'tg 1/5/2010 17:14' prior: 0!
FormsBuilder is a generic algorithm for placing elements that have size in a grid that can be scaled both horizontally and vertically.

In Mondrian it has two applications:

1. for building a complex shape. In this case x:y:add: accepts a Shape. You can see an example in UMLClassDiagram>>umlShape. You obtain the shape from the builder by sending #shape. A shape created by a FormsBuilder is an instance of FormsShape.

2. for laying out the nodes in a graph. This is achieved by passing MONodes in x:y:add: and by sending asLayout to the FormsBuilder and as you see in the MOMatrixRenderer.!

!MOFormsBuilder methodsFor: 'accessing' stamp: 'AlexandreBergel 11/22/2011 10:26'!
asLayout
	
	^MOFormsLayout on: self! !

!MOFormsBuilder methodsFor: 'accessing' stamp: 'AlexandreBergel 8/2/2011 17:11'!
asShape
	
	| allShapes |	
	allShapes := OrderedCollection new.
	self shapes
		do: [:component | allShapes addAll: component collectAllShapes].	
	^MOFormsShape on: self! !

!MOFormsBuilder methodsFor: 'utils' stamp: ' 22/10/08 14:21'!
at: aX and: aY
	"Utility method that makes other methods easier to read."
	
	^self components
		select: [:component | component gridX = aX and: [component gridY = aY]]! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:56'!
bottom
	
	alignment
		removeLast;
		addLast: #bottom! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: 'AlexandreBergel 4/11/2011 10:49'!
boundsOf: aShape for: graphElement
	"Return the position of a shape for a given graphElement"
	
	| aComponent originX originY columnNumber rowNumber columnWidth rowHeight maxOriginX maxOriginY s componentWidth componentHeight |

	"pref says that the image inside sets the width, the shape has precedence"
	"grow says that when you resize the complete shape, the column will resize proportionally"
	
	originX := 0.
	originY := 0.
	columnWidth := 0.
	rowHeight := 0.

	aComponent := self shapes 
					detect: [:each | each shape == aShape] 
					ifNone: [ self error: 'Cannot determine the bounds of a shape that I do not contain' ].
	columnNumber := aComponent gridX.
	rowNumber := aComponent gridY.

	componentWidth := aComponent widthFor: graphElement.
	componentHeight := aComponent heightFor: graphElement.


	1 to: self numberOfColumns do: [:column | 
		(self at: column and: rowNumber) do: [:comp |
			s := (comp heightFor: graphElement) / comp rowSpan.
			"s := (comp heightFor: (comp figure)) / comp rowSpan."
			rowHeight := rowHeight max: s]].
"	componentHeight := rowHeight."
	
	1 to: self numberOfRows do: [:row | 
		(self at: columnNumber and: row) do: [:comp |
			s := (comp widthFor: graphElement) / comp columnSpan.
			"s := (comp widthFor: (comp figure)) / comp columnSpan."
			columnWidth := columnWidth max: s]].
"	componentWidth := columnWidth."

	((self columnAlignmentAt: columnNumber) == #fill)
		ifTrue: [
			componentWidth := columnWidth.
			(columnNumber + 1) to: (columnNumber + aComponent columnSpan -1 ) do: [:cNumber |
				columnWidth := 0.
				1 to: self numberOfRows do: [:row | 
						(self at: cNumber and: row) do: [:comp |
							"s := (comp widthFor: (comp figure)) / comp columnSpan."
							s := (comp widthFor: graphElement) / comp columnSpan.
							columnWidth := columnWidth max: s]].
				componentWidth := componentWidth + columnWidth]].

	((self rowAlignmentAt: rowNumber) == #fill)
		ifTrue: [	
			componentHeight := rowHeight.
			rowNumber + 1 to: (rowNumber + aComponent rowSpan - 1) do: [:rNumber |
				rowHeight := 0.
				1 to: self numberOfColumns do: [:column | 
						(self at: column and: rNumber) do: [:comp |
							s := (comp heightFor: graphElement) / comp rowSpan.
							"s := (comp heightFor: (comp figure)) / comp rowSpan."
							rowHeight := rowHeight max: s]].
				componentHeight := componentHeight + rowHeight]].


	1 to: columnNumber -1 do: [:column | 
		maxOriginX := 0.
		1 to: self numberOfRows do: [:row | 
			(self at: column and: row) do: [:comp |
				s := (comp widthFor: graphElement) / comp columnSpan.
				"s := (comp widthFor: (comp figure)) / comp columnSpan."
				maxOriginX := maxOriginX max: s]].
		originX := originX + maxOriginX].

	1 to: rowNumber - 1 do: [:row |
		maxOriginY := 0.
		1 to: self numberOfColumns do: [:column | 
			(self at: column and: row) do: [:comp | 
				"s := (comp heightFor: (comp figure)) / comp rowSpan."
				s := (comp heightFor: graphElement) / comp rowSpan.
				maxOriginY := maxOriginY max: s]].
		originY := originY + maxOriginY].

	
	((self columnAlignmentAt: columnNumber) = #right and: [aComponent columnSpan = 1])
		ifTrue: [originX :=  originX + (columnWidth - (aComponent widthFor: graphElement))].
	((self columnAlignmentAt: columnNumber) = #center and: [aComponent columnSpan = 1])
		ifTrue: [originX :=  originX + (columnWidth / 2.0 - ((aComponent widthFor: graphElement) / 2.0))].
	((self rowAlignmentAt: rowNumber) = #bottom and: [aComponent rowSpan = 1])
		ifTrue: [originY :=   originY + (rowHeight - (aComponent heightFor: graphElement))].
	((self rowAlignmentAt: rowNumber) = #center and: [aComponent rowSpan = 1])
		ifTrue:	[originY :=  originY + (rowHeight / 2.0 - ((aComponent heightFor: graphElement) / 2.0))].
	
	^ (originX @ originY) extent: (componentWidth @ componentHeight)! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:56'!
center
	
	alignment
		removeLast;
		addLast: #center! !

!MOFormsBuilder methodsFor: 'testing' stamp: 'Alexandre.Bergel 2/1/2009 10:06'!
checkIfOK: aBounds
	"This method is invoked when an element is added into the builder. The area associated with this element is aBounds.
	Prior to add it, we need this to ensure that the element to add is not out of bound (i.e., that the size of the builder is properly 
	defined). This is the purpose of this method."
	| raiseError |
	raiseError := false.
	((aBounds origin x + aBounds width) > (columnDefaultSize size + 1))
		ifTrue: [ raiseError := true].

	((aBounds origin y + aBounds height) > (rowDefaultSize size + 1))
		ifTrue: [ raiseError := true].
		
		
	raiseError ifTrue: [ self error: 'You trying to add an out of bounds element. Adjust the size of the builder' ]! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: ' 22/10/08 14:21'!
column
	
	alignment := columnAlignment.
	resizeBehaviour := columnResizeBehaviour.
	defaultSize := columnDefaultSize.
	alignment addLast: #none.
	resizeBehaviour addLast: #none.
	defaultSize addLast: #pref! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
columnAlignmentAt: anIndex
	
	^columnAlignment at: anIndex! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
columnDefaultSizeAt: anIndex
	
	^columnDefaultSize at: anIndex! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
columnResizeBehaviourAt: anIndex
	
	^columnResizeBehaviour at: anIndex! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
columns
	
	^self numberOfColumns! !

!MOFormsBuilder methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2010 09:33'!
columns: aString
	| columns |
	"Get all the column definitions"
	columns := aString copyReplaceAll: ' ' with: ''.
	columns := columns subStrings: ','.
	columns
		do: [ :column | 
			| tokens |
			self column.
			tokens := column subStrings: ':'.
			tokens
				do: [ :token | 
					(token findString: 'px' startingAt: 1) = 0
						ifTrue: [ self perform: token asSymbol ]
						ifFalse: [ self size: (token copyFrom: 1 to: token size - 2) asNumber ] ] ]! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
components
	
	components isNil ifTrue: [components := OrderedCollection new].
	^components! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:56'!
fill
	
	alignment
		removeLast;
		addLast: #fill! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: 'AlexandreBergel 4/7/2011 12:48'!
graphElements
	
	graphElements isNil ifTrue: [graphElements := OrderedCollection new].
	^graphElements! !

!MOFormsBuilder methodsFor: 'utils' stamp: ' 22/10/08 14:21'!
gridDo: aBlock
	"Utility method that makes other methods easier to read."
	
	1
		to: self numberOfRows
		do:
			[:row | 
			1
				to: self numberOfColumns
				do: [:column | aBlock value: row value: column]]! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:56'!
grow
	
	resizeBehaviour
		removeLast;
		addLast: #grow! !

!MOFormsBuilder methodsFor: 'layout-apply' stamp: 'AlexandreBergel 4/9/2011 08:52'!
heightsFor: graphElement
	
	| myHeights count availableSpace gapSize |
	self initComponentsFor: graphElement.
	myHeights := self initialHeights.	"Iterate over all components that only span a single row"
	self
		gridDo:
			[:row :column | 
			(self at: column and: row)
				do:
					[:component | 
					component rowSpan = 1
						ifTrue:
							[(self rowDefaultSizeAt: row) = #pref
								ifTrue:
									[myHeights
										at: row
										put: ((component heightFor: graphElement)  max: (myHeights at: row))].
							((self rowDefaultSizeAt: row) isKindOf: Number)
								ifTrue:
									[myHeights
										at: row
										put: (self rowDefaultSizeAt: row)]]]].	"Try to satisfy the wishes of the multi-row components"
	self
		gridDo:
			[:row :column | 
			(self at: column and: row)
				do:
					[:component | 
					"Calculate row heights"
					count := 0.
					availableSpace := 0.0.
					row
						to: row + component rowSpan - 1
						do:
							[:index | 
							availableSpace := availableSpace + (myHeights at: index).
							(self rowResizeBehaviourAt: index) = #grow
								ifTrue: [count := count + 1]].
					gapSize := (0.0 max: (component heightFor: graphElement)  - availableSpace)
						/ (count max: 1).
					row
						to: row + component rowSpan - 1
						do:
							[:index | 
							(self rowResizeBehaviourAt: index) = #grow
								ifTrue:
									[myHeights
										at: index
										put: (myHeights at: index) + gapSize]]]].
	^myHeights! !

!MOFormsBuilder methodsFor: 'utils' stamp: 'AlexandreBergel 4/9/2011 08:44'!
initComponentsFor: graphElement
	"Initialize shapes"
	
	graphElement
		ifNotNil: [ self shapes do: [:component | graphElement attributeAt: #shapeAdaptor put:  component  ] ].
	self graphElements do: [ :component | component initializeBounds ]! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: 'Alexandre Bergel 7/26/2010 14:59'!
initialHeights
	"Here we take rows into account that have a pixel height"

	| heights |
	heights := (Array new: self numberOfRows) atAllPut: 0.0.
	1 to: self numberOfRows do: [ :index | 
		((self rowDefaultSizeAt: index) isKindOf: Number)
			ifTrue: [ heights at: index put: (self rowDefaultSizeAt: index) ] ].
	^ heights! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: 'Alexandre Bergel 7/26/2010 14:59'!
initialWidths
	"Here we take columns into account that have a pixel width"

	| widths |
	widths := (Array new: self numberOfColumns) atAllPut: 0.0.
	1 to: self numberOfColumns do: [ :index | 
		((self columnDefaultSizeAt: index) isKindOf: Number)
			ifTrue: [ widths at: index put: (self columnDefaultSizeAt: index) ] ].
	^ widths! !

!MOFormsBuilder methodsFor: 'initialize-release' stamp: ' 22/10/08 14:21'!
initialize
	
	columnAlignment := OrderedCollection new.
	columnResizeBehaviour := OrderedCollection new.
	columnDefaultSize := OrderedCollection new.
	rowAlignment := OrderedCollection new.
	rowResizeBehaviour := OrderedCollection new.
	rowDefaultSize := OrderedCollection new! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:57'!
left
	
	alignment
		removeLast;
		addLast: #left! !

!MOFormsBuilder methodsFor: 'private' stamp: ' 22/10/08 14:21'!
numberOfColumns
	
	^columnAlignment size! !

!MOFormsBuilder methodsFor: 'private' stamp: ' 22/10/08 14:21'!
numberOfRows
	
	^rowAlignment size! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: 'AlexandreBergel 4/15/2011 22:47'!
positionOf: aComponent
	"Return the position of the component in the grid"

	| el |
	el := self components detect: [:c | c component == aComponent ].
	^ el gridX @ el gridY! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:57'!
pref
	
	defaultSize
		removeLast;
		addLast: #pref! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:56'!
right
	
	alignment
		removeLast;
		addLast: #right! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: ' 22/10/08 14:21'!
row
	
	alignment := rowAlignment.
	resizeBehaviour := rowResizeBehaviour.
	defaultSize := rowDefaultSize.
	alignment addLast: #none.
	resizeBehaviour addLast: #none.
	defaultSize addLast: #pref! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
rowAlignmentAt: anIndex
	
	^rowAlignment at: anIndex! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
rowDefaultSizeAt: anIndex
	
	^rowDefaultSize at: anIndex! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
rowResizeBehaviourAt: anIndex
	
	^rowResizeBehaviour at: anIndex! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
rows
	
	^self numberOfRows! !

!MOFormsBuilder methodsFor: 'accessing' stamp: 'AlexandreBergel 9/5/2010 09:33'!
rows: aString
	| rows |
	"Get all the column definitions"
	rows := aString copyReplaceAll: ' ' with: ''.
	rows := rows subStrings: ','.
	rows
		do: [ :row | 
			| tokens |
			self row.
			tokens := row subStrings: ':'.
			tokens
				do: [ :token | 
					(token findString: 'px' startingAt: 1) = 0
						ifTrue: [ self perform: token asSymbol ]
						ifFalse: [ self size: (token copyFrom: 1 to: token size - 2) asNumber ] ] ]! !

!MOFormsBuilder methodsFor: 'layout-apply' stamp: 'AlexandreBergel 12/10/2010 12:24'!
runWithWidths: myWidths heights: myHeights
	
	| x y width height |

	self
		gridDo:
			[:row :column | 
			(self at: column and: row)
				do:
					[:component | 
					"Move the shape to the proper position"
					x := (myWidths
						copyFrom: 1
						to: column - 1)
						inject: 0
						into: [:sum :next | sum + next].
					y := (myHeights
						copyFrom: 1
						to: row - 1)
						inject: 0
						into: [:sum :next | sum + next].	"By default we align #left"
					component translateTo: x @ y.	"Update the width"
					width := (myWidths
						copyFrom: column
						to: column + component columnSpan - 1)
						inject: 0.0
						into: [:sum :next | sum + next].	"If we use px size we may need to crop the size"
					(((self columnDefaultSizeAt: column) isKindOf: Number)
						and: [component columnSpan = 1])
						ifTrue: [component width: (width min: component width)].	"Update the width and position of the shape based on the alignment"
			
					((self columnAlignmentAt: column) = #fill
						or: [component columnSpan > 1]) ifTrue: [component width: width].	"(columnAlignment: column) = #left ifTrue: [We don't need to do anything]."
					((self columnAlignmentAt: column) = #right
						and: [component columnSpan = 1])
						ifTrue: [component translateBy: (width - component width) @ 0].
					((self columnAlignmentAt: column) = #center
						and: [component columnSpan = 1])
						ifTrue:
							[component translateBy: (width / 2.0 - (component width / 2.0)) @ 0].	"Update the height"
					height := (myHeights
						copyFrom: row
						to: row + component rowSpan - 1)
						inject: 0.0
						into: [:sum :next | sum + next].	"If we use px size we may need to crop the size"
					(((self rowDefaultSizeAt: row) isKindOf: Number)
						and: [component rowSpan = 1])
						ifTrue: [component height: (height min: component height)].	"Update the height and position of the shape based on the alignment"
					((self rowAlignmentAt: row) = #fill or: [component rowSpan > 1])
						ifTrue: [component height: height].	"(rowAlignment: row) = #top ifTrue: [We don't need to do anything]."
					((self rowAlignmentAt: row) = #bottom and: [component rowSpan = 1])
						ifTrue: [component translateBy: 0 @ (height - component height)].
					((self rowAlignmentAt: row) = #center and: [component rowSpan = 1])
						ifTrue:
							[component translateBy: 0 @ (height / 2.0 - (component height / 2.0))]]]! !

!MOFormsBuilder methodsFor: 'accessing' stamp: ' 22/10/08 14:21'!
shape
	
	^self asShape! !

!MOFormsBuilder methodsFor: 'accessing - data' stamp: ' 22/10/08 14:21'!
shapes
	
	shapes isNil ifTrue: [shapes := OrderedCollection new].
	^shapes! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:56'!
size: aNumber
	
	defaultSize
		removeLast;
		addLast: aNumber! !

!MOFormsBuilder methodsFor: 'layout-define' stamp: 'Alexandre.Bergel 8/30/2009 15:56'!
top
	
	alignment
		removeLast;
		addLast: #top! !

!MOFormsBuilder methodsFor: 'layout-apply' stamp: 'AlexandreBergel 4/9/2011 08:51'!
widthsFor: graphElement
	
	| myWidths count availableSpace gapSize |
	self initComponentsFor: graphElement.
	myWidths := self initialWidths.	"Iterate over all components that only span a single column"
	self
		gridDo:
			[:row :column | 
			(self at: column and: row)
				do:
					[:component | 
					component columnSpan = 1
						ifTrue:
							[(self columnDefaultSizeAt: column) = #pref
								ifTrue:
									[myWidths
										at: column
										put: ((component widthFor: graphElement)  max: (myWidths at: column))].
							((self columnDefaultSizeAt: column) isKindOf: Number)
								ifTrue:
									[myWidths
										at: column
										put: (self columnDefaultSizeAt: column)]]]].	"Try to satisfy the wishes of multi-column components"
	self
		gridDo:
			[:row :column | 
			(self at: column and: row)
				do:
					[:component | 
					"Calculate column widths"
					count := 0.
					availableSpace := 0.0.
					column
						to: column + component columnSpan - 1
						do:
							[:index | 
							availableSpace := availableSpace + (myWidths at: index).
							(self columnResizeBehaviourAt: index) = #grow
								ifTrue: [count := count + 1]].
					gapSize := (0.0 max: (component widthFor: graphElement) - availableSpace)
						/ (count max: 1).
					column
						to: column + component columnSpan - 1
						do:
							[:index | 
							(self columnResizeBehaviourAt: index) = #grow
								ifTrue:
									[myWidths
										at: index
										put: (myWidths at: index) + gapSize]]]].
	^myWidths! !

!MOFormsBuilder methodsFor: 'accessing' stamp: 'AlexandreBergel 9/6/2010 18:29'!
x: aX y: aY
	
	^ (self components detect: [:c | c gridX = aX and: [ c gridY = aY ]]) component! !

!MOFormsBuilder methodsFor: 'accessing' stamp: ' 22/10/08 14:21'!
x: aX y: aY add: anObject
	
	self x: aX y: aY w: 1 h: 1 add: anObject! !

!MOFormsBuilder methodsFor: 'accessing' stamp: ' 22/10/08 14:21'!
x: aX y: aY h: aH add: anObject
	
	self x: aX y: aY w: 1 h: aH add: anObject! !

!MOFormsBuilder methodsFor: 'accessing' stamp: ' 22/10/08 14:21'!
x: aX y: aY w: aW add: anObject
	
	self x: aX y: aY w: aW h: 1 add: anObject! !

!MOFormsBuilder methodsFor: 'accessing' stamp: 'AlexandreBergel 4/7/2011 12:47'!
x: aX y: aY w: aW h: aH add: aSomething
	
	| component |
	self checkIfOK: (aX@aY extent: (aW@aH)).
	self assert: [ (aSomething isKindOf: MOViewRenderer) not ].
	
	"We check we do not add more than one children shape. Note that this restriction may diseappar in the future"
	(aSomething isShape and: [ aSomething isChildShape ])
		ifTrue: [ | children | 
				 children := (self shapes select: [:v | v component isChildShape]).
				 children size >= 1 ifTrue: [
					(children anySatisfy: [:v | v gridX ~= aX or: [ v gridY ~= aY]])
						ifTrue: [ self error: 'no more than one MOChildrenShape may be added in a builder' ]
				 ]
				  ].
	
	aSomething isShape
		ifTrue:
			[ component := MOShapeAdaptor on: aSomething.
			  aSomething disableCache.
			 self shapes add: component ]
		ifFalse:
			[ component := MOFigureAdaptor on: aSomething.
			 self graphElements add: component ].
	self components add: component.
	component gridX: aX.
	component gridY: aY.
	component columnSpan: aW.
	component rowSpan: aH! !

!Object methodsFor: '*mondrian-complexshape' stamp: 'AlexandreBergel 4/28/2011 18:52'!
isShape
	"I am not a Mondrian shape. This method is intended to be overriden"
	
	^ false! !
